#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <bios.h>

#define _TCK 0x80
#define _TMS 0x40
#define _TDO 0x01

unsigned int oport = 0x378;  /* Out reg of LPT port */
unsigned int iport = 0x379;  /* Status reg of LPT port */

static unsigned int state;

inline static int TDI(void) {
	return (inport(iport) & 0x80) != 0x80;
}

inline static void TCK(int x) {
	if(x) state |= _TCK; else state &= ~_TCK;
   outport(oport, state);
}
inline static void TCK_1_0(void) {
/*
	state |=  _TCK; outport(oport, state);
   state &= ~_TCK; outport(oport, state);
*/
	outport(oport, state | _TCK); /* Imply TCK=0*/
   outport(oport, state);
}


inline static void TMS(int x) {
	if(x) state |= _TMS; else state &= ~_TMS;
   outport(oport, state);
}

inline static void TDO(int x) {
	if(x) state |= _TDO; else state &= ~_TDO;
   outport(oport, state);
}

void ResetJTAG(void) {
   TMS(1); /* Reset of controller JTAG interface */
   TCK(0); for(int i=0; i<5; i++) { TCK_1_0(); }
   TMS(0); TCK_1_0(); /* -> Run-Test/Idle */
   TMS(1); TCK_1_0(); /* -> Select-DR-Scan */
}

void Command(int x) {

   TMS(1); TCK_1_0(); /* -> Select-IR-Scan */
   TMS(0); TCK_1_0(); /* -> Capture-IR */
   /*  0*/ TCK_1_0();

   TDO(x&1); TCK_1_0();
   TDO(x&2); TCK_1_0();
   TDO(x&4); TCK_1_0();
		TMS(1); /* Prepare to break from loop */
   TDO(x&8); TCK_1_0();
   /*  1*/ TCK_1_0(); /* -> Update-IR      */
   /*  1*/ TCK_1_0(); /* -> Select-DR-Scan */
}


void ExenData(unsigned char *p, int bits) {

   TMS(0); TCK_1_0(); /* -> Capture-DR     */
   /*  0*/ TCK_1_0(); /* -> Shift-DR       */

   for(int i=0; i<(bits-1); i++) {
		TDO(*p); TCK(1); *p++ = TDI(); TCK(0);
   }
   TMS(1); /*  Prepare to break from loop + last bit */
   TDO(*p); TCK(1); *p = TDI(); TCK(0); /* -> Exit1-DR  */

   /*  1*/ TCK_1_0(); /* -> Update-DR */
   /*  1*/ TCK_1_0(); /* -> Select-DR-Scan */
}

void Step(unsigned char buf[], unsigned long x){

   buf[0]=1; // ABE=1
   for(int i=1;i<121;i++) buf[i]=0;
   buf[41]=0;  //nMREQ
   buf[55]=1; // Nenout
   buf[51]=buf[49]=buf[42]=buf[33]=1;

   for(int i=1; i<33; i++) {
			buf[i]= x & 1;
			x>>=1;
   }

}

/**************************************************************************/
void main(void) {

  unsigned char buf[121];

   directvideo = 1;
   clrscr();
	gotoxy(1, 23);
	cprintf("Getting memory context!");


   ResetJTAG();


   Command(0xE);  /* Load IDCODE */
   ExenData(buf, 32);

   long ID=0;
   for(int i=0;i<32;i++) { ID+=buf[31-i] + ID; }
   gotoxy(1, 25);
   cprintf("JTAG ID code:%08lX", ID);


	Command(0x3);  /* Load SAMPLE/PRELOAD */
	Step(buf,0L);
	ExenData(buf, 121);

	Command(0);  /* Load EXTEST */
	Step(buf,0L);
	ExenData(buf, 121);

//***********
   FILE *file;
   unsigned long adr=0;  // Start adr
	Command(0);  /* Load EXTEST */

   file=fopen("data.bin","wb");
   if(file) {
   		do {

		Step(buf,adr+4L);
		ExenData(buf, 121);

		unsigned long din;
                din=0;
		for(int i=58; i<121; i+=2) din+=buf[i] + din;

		if((adr & 0xFF)==0) {
					gotoxy(40, 25);
					cprintf("%08lX:%08lX", adr, din);
		}

		din=(din>>24) | ((din&0x00FF0000L)>>8) | ((din&0x0000FF00L)<<8) | (din<<24);
		fwrite(&din, sizeof(din), 1, file);

		adr += 4;

   } while(!bioskey(1));

   fclose(file);
}

}
